import os

from dtk.utils.core.DTKConfigBuilder import DTKConfigBuilder
from simtools.SetupParser import SetupParser
from simtools.ExperimentManager.ExperimentManagerFactory import ExperimentManagerFactory


# STEPS TO RUN CATALYST ON SLURM CLUSTERS
#
# 1. mkdir <new_workding_dir>
# 2. cd <new_working_dir>
# 3. cp <simtools.ini_for_simulation_to_explore> .
# 4. Change input_root in your simtools.ini run block to (no quotes): "input_root = input"
# 5. ln -s <simulation_to_explore_directory> input
# 6. <put this python file in your current directory>
# 7. <Edit the notated section below as needed>
# 8. Example catalyst execution with three population scaling factors, [] indicates the base case for comparison:
# dtk catalyst -s popscaling -M "[0.005],0.004,0.002" example_catalyst_slurm.py
# 9. When your jobs are complete, re-run catalyst with the experiment id added to generate the report, e.g.:
# dtk catalyst -s popscaling -M "[0.005],0.004,0.002" example_catalyst.py -id 2020_01_31_12_49_43_062204
# (make sure to answer Y for the report to be generated, R for resume (jobs are complete)

# >>> BEGIN USER EDIT >>>

SetupParser.default_block = "NYUCLUSTER"

# Name of the catalyst experiment to run
exp_name = 'catalyst_comparison-hiv-slurm-example'

# Filenames in simulation being swept with catalyst
config_filename = 'config.json'
campaign_filename = 'campaign_Nyanza.json'

# <<< END USER EDIT <<<

input_dir = 'input'
config_file = os.path.join(input_dir, config_filename)
campaign_file = os.path.join(input_dir, campaign_filename)

kwargs = {}
cb = DTKConfigBuilder.from_files(config_name=config_file, campaign_name=campaign_file, **kwargs)


# when run with 'dtk catalyst', run_sim_args['exp_name'] will have additional information appended.
run_sim_args =  {
    'exp_name': exp_name,
    # 'exp_builder': builder, # users may have created one; this will be overridden in 'dtk catalyst'
    'config_builder': cb
}

#if __name__ == "__main__":
#    SetupParser.init()
#    exp_manager = ExperimentManagerFactory.init()
#    exp_manager.run_simulations(**run_sim_args)
