import json
import os

from simtools.DataAccess.Schema import Experiment
from simtools.SetupParser import SetupParser


class CLUSTERExperiment:
    def __init__(self, exp_id, exp_name, suite_id, path):
        self.exp_id = exp_id
        self.id = exp_id  # To mimic both COMPS experiment objects and local experiment objects
        self.exp_name = exp_name
        self.suite_id = suite_id
        self.path = path

    @classmethod
    def from_path(cls, path):
        root_dir = os.path.basename(path)
        try:
            exp_name, exp_id = Experiment.split_id(id=root_dir)
        except:
            print('FAILED to parse exp_name/exp_id')
            return None
        try:
            suite_file = os.path.join(path, 'suite.json')
            with open(suite_file, 'r') as f:
                suite_id = json.load(f)['id']
        except:
            print('FAILED to read suite file')
            return None
        return cls(exp_id=exp_id, exp_name=exp_name, suite_id=suite_id, path=path)


def exps_for_suite_id(suite_id):
    experiments = []
    experiments_directory = SetupParser.get('sim_root')
    for item in os.listdir(experiments_directory):
        experiment_directory = os.path.join(experiments_directory, item)
        if os.path.isdir(experiment_directory):
            experiment = CLUSTERExperiment.from_path(experiment_directory)
            if experiment is not None:
                if (experiment.suite_id == suite_id):
                    experiments.append(experiment)
    return experiments
